==> jabber-transport-0.9.tar.gz: jabber-transport-0.9/src/svc/telnet/svc_telnet.c <==
#include "jserver.h"

/* this is a big hack for debugging/testing, someone PLEASE turn it something useful :) */

#define PHASE_LOGIN 1
#define PHASE_PASS 2
#define PHASE_RES 3
#define PHASE_NORM 4
#define PHASE_MSG 5

typedef struct teldata_struct
{
    int phase, initflag;
    char *login;
    char *pass;
    char *res;
    char *message;
    jid to;
    session s;
    jid id;
    thread t;
} *teldata, _teldata;

int svc_telnet_auth(teldata td)
{
    xmlnode x, q;
    int ret;

    x = xmlnode_new_tag("iq");
    xmlnode_put_attrib(x, "type", "set");
    q = xmlnode_insert_tag(x, "query");
    xmlnode_put_attrib(q, "xmlns", NS_AUTH);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "username"),td->login,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "password"),td->pass,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "resource"),td->res,-1);

    log_debug(ZONE,"auth %s",xmlnode2str(x));
    ret = js_service_prescreen(jpacket_new(x));
    log_debug(ZONE,"auth %d ret %s",ret, xmlnode2str(x));

    xmlnode_free(x);

    if(ret == S_AUTHED)
        return 1;
    else
        return 0;
}

int svc_telnet_reg(teldata td)
{
    xmlnode x, q;
    int ret;
    jpacket p;

    x = xmlnode_new_tag("iq");
    xmlnode_put_attrib(x, "type", "set");
    q = xmlnode_insert_tag(x, "query");
    xmlnode_put_attrib(q, "xmlns", NS_REGISTER);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "username"),td->login,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "password"),td->pass,-1);

    log_debug(ZONE,"register %s",xmlnode2str(x));
    p = jpacket_new(x);
    js_service_prescreen(p);

    jpacket_reset(p);
    ret = jpacket_subtype(p);
    log_debug(ZONE,"register %d ret %s",ret, xmlnode2str(x));
    xmlnode_free(x);

    if(ret == JPACKET__RESULT)
        return 1;
    else
        return 0;
}

void svc_telnet_message(teldata td)
{
    xmlnode x, body;
    jid id;
    jpacket p;


    x = xmlnode_new_tag("message");
    xmlnode_put_attrib(x, "to", jid_full(td->to));
    body = xmlnode_insert_tag(x, "body");
    xmlnode_insert_cdata(body,td->message,strlen(td->message));
    free(td->message);

    log_debug(ZONE,"sending %s",xmlnode2str(x));

    p = jpacket_new(x);
    js_psend(td->s->mp_from,p);
}

void svc_telnet_send(session s, jpacket p, void *arg)
{
    teldata td = (teldata)arg;

    if(p == NULL) return; /* XXX session ended, do */

    log_debug(ZONE,"send %s",xmlnode2str(p->x));
    tstream_write(td->t, "\nIncoming Message!\nFrom: ", -1);
    tstream_write(td->t, jid_full(p->from), -1);
    tstream_write(td->t, "\n", -1);
    tstream_write(td->t, xmlnode_get_tag_data(p->x,"body"), -1);
    tstream_write(td->t, "\njabber>", -1);
}

void svc_telnet_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    teldata td = (teldata)(ts->data);
 
    log_debug(ZONE,"svc_telnet_cleanup");
    if(td->s != NULL)
        js_session_end(td->s,"Disconnected");
}

void svc_telnet_read(thread t, void *buffer, int bufsz)
{
    char *eol, *space, *newmsg;
    tstream ts = (tstream)(t->data);
    teldata td = (teldata)(ts->data);

    if(td->initflag)
    {
        pth_cleanup_push(svc_telnet_cleanup, (void *)t);
        td->initflag = 0;
    }

    if(bufsz <= 0) return;

    if((eol = strstr(buffer, "\r")) != NULL) *eol = '\0';
    if((eol = strstr(buffer, "\n")) != NULL) *eol = '\0';

    log_debug(ZONE,"phase %d data %s",td->phase, buffer);

    switch(td->phase)
    {
        case PHASE_LOGIN:
            td->login = pstrdup(t->p, buffer);
            tstream_write(t, "password: ", -1);
            td->phase = PHASE_PASS;
            break;
        case PHASE_PASS:
            td->pass = pstrdup(t->p, buffer);
            tstream_write(t, "resource[telnet]: ", -1);
            td->phase = PHASE_RES;
            break;
        case PHASE_RES:
            if(strlen(buffer) > 0)
                td->res = pstrdup(t->p, buffer);
            else
                td->res = pstrdup(t->p, "telnet");

            if(svc_telnet_auth(td))
            {
                tstream_write(t, "Heya bud! :)\n", -1);
            }else if(strcmp(td->res, "register") == 0 && svc_telnet_reg(td) && svc_telnet_auth(td)){
                tstream_write(t, "Account Created\n", -1);
            }else{
                tstream_write(t, "Shea, right!\nlogin: ", -1);
                td->phase = PHASE_LOGIN;
                break;
            }
            /* we're authed! */
            tstream_write(t, "\njabber> ", -1);
            td->phase = PHASE_NORM;
            td->id = jid_new(t->p,xmlnode_get_data(js_config("names/default")));
            jid_set(td->id, td->login, JID_USER);
            jid_set(td->id, td->res, JID_RESOURCE);
            td->s = js_session_new(td->id,svc_telnet_send,(void *)td);
            break;
        case PHASE_NORM:
            space = strstr(buffer," ");
            if(space != NULL)
            {
                *space = '\0';
                ++space;
            }
            if(strcmp(buffer, "quit") == 0)
            {
                tstream_write(t, "Adios Amigo\n\nlogin:", -1);
                js_session_end(td->s,space);
                td->phase = PHASE_LOGIN;
                break;
            }
            if(strcmp(buffer, "message") == 0 && (td->to = jid_new(t->p, space)) != NULL)
            {
                tstream_write(t, "Composing message to ", -1);
                tstream_write(t, space, -1);
                tstream_write(t, ":\n", 2);
                td->phase = PHASE_MSG;
                td->message = malloc(1);
                *(td->message) = '\0';
                break;
            }
            tstream_write(t, "jabber> ", -1);
            break;
        case PHASE_MSG:
            if(strcmp(buffer, ".") == 0)
            {
                svc_telnet_message(td);
                td->phase = PHASE_NORM;
                tstream_write(t, "jabber> ", -1);
                break;
            }
            newmsg = malloc(strlen(td->message) + strlen(buffer) + 2);
            *newmsg = '\0';
            strcat(newmsg,td->message);
            strcat(newmsg," ");
            strcat(newmsg,buffer);
            free(td->message);
            td->message = newmsg;
            break;
        default:
            tstream_write(t, "I suck! I'm dumb!\n", -1);
    }
}

void svc_telnet_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
    teldata newd;
    tstream ts;

    log_debug(ZONE,"svc_telnet connection established (fd: %d, ip: %s, port: %d)\n",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    new = tstream_new(sock, inet_ntoa(sa.sin_addr), svc_telnet_read, NULL, NULL);
    ts = (tstream)(new->data);
    newd = pmalloc(new->p, sizeof(_teldata));
    newd->phase = PHASE_LOGIN;
    newd->initflag = 1;
    newd->t = new;
    newd->s = NULL;
    ts->data = (void *)newd;
    tstream_write(new, "Welcome!\nlogin: ", -1);
}

void svc_telnet()
{
    log_debug(ZONE,"svc_telnet startup");
    tlisten_new(5228, NULL, svc_telnet_connect, NULL);
}

/* for fun, 3:15 am Jan 28th 2000:
[jer@atreju jabber-transport]$ telnet localhost 5228
Trying 127.0.0.1...
Connected to localhost.
Escape character is '^]'.
Welcome!
login: jer
password: jabber
resource[telnet]:
Heya bud! :)
jabber> message jer@atreju.jeremie.com/telnet
Composing message to jer@atreju.jeremie.com/telnet:
will?
this be the first 0.9 message? :)
.
jabber>
Incoming Message!
From: jabber:jer@atreju.jeremie.com/telnet
 will? this be the first 0.9 message? :)
jabber>
*/

==> jabber-transport-1.0.tar.gz: jabber-transport-1.0/src/svc/telnet/svc_telnet.c <==
#include "jserver.h"

/* this is a big hack for debugging/testing, someone PLEASE turn it something useful :) */

#define PHASE_LOGIN 1
#define PHASE_PASS 2
#define PHASE_RES 3
#define PHASE_NORM 4
#define PHASE_MSG 5

typedef struct teldata_struct
{
    int phase, initflag;
    char *login;
    char *pass;
    char *res;
    char *message;
    jid to;
    session s;
    jid id;
    thread t;
} *teldata, _teldata;

int svc_telnet_auth(teldata td)
{
    xmlnode x, q;
    int ret;

    x = xmlnode_new_tag("iq");
    xmlnode_put_attrib(x, "type", "set");
    q = xmlnode_insert_tag(x, "query");
    xmlnode_put_attrib(q, "xmlns", NS_AUTH);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "username"),td->login,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "password"),td->pass,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "resource"),td->res,-1);

    log_debug(ZONE,"auth %s",xmlnode2str(x));
    ret = js_service_prescreen(jpacket_new(x));
    log_debug(ZONE,"auth %d ret %s",ret, xmlnode2str(x));

    xmlnode_free(x);

    if(ret == S_AUTHED)
        return 1;
    else
        return 0;
}

int svc_telnet_reg(teldata td)
{
    xmlnode x, q;
    int ret;
    jpacket p;

    x = xmlnode_new_tag("iq");
    xmlnode_put_attrib(x, "type", "set");
    q = xmlnode_insert_tag(x, "query");
    xmlnode_put_attrib(q, "xmlns", NS_REGISTER);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "username"),td->login,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "password"),td->pass,-1);

    log_debug(ZONE,"register %s",xmlnode2str(x));
    p = jpacket_new(x);
    js_service_prescreen(p);

    jpacket_reset(p);
    ret = jpacket_subtype(p);
    log_debug(ZONE,"register %d ret %s",ret, xmlnode2str(x));
    xmlnode_free(x);

    if(ret == JPACKET__RESULT)
        return 1;
    else
        return 0;
}

void svc_telnet_message(teldata td)
{
    xmlnode x, body;
    jpacket p;


    x = xmlnode_new_tag("message");
    xmlnode_put_attrib(x, "to", jid_full(td->to));
    body = xmlnode_insert_tag(x, "body");
    xmlnode_insert_cdata(body,td->message,strlen(td->message));
    free(td->message);

    log_debug(ZONE,"sending %s",xmlnode2str(x));

    p = jpacket_new(x);
    js_session_from(td->s,p);
}

void svc_telnet_send(session s, jpacket p, void *arg)
{
    teldata td = (teldata)arg;

    if(p == NULL) return; /* XXX session ended, do */

    log_debug(ZONE,"send %s",xmlnode2str(p->x));
    tstream_write(td->t, "\nIncoming Message!\nFrom: ", -1);
    tstream_write(td->t, jid_full(p->from), -1);
    tstream_write(td->t, "\n", -1);
    tstream_write(td->t, xmlnode_get_tag_data(p->x,"body"), -1);
    tstream_write(td->t, "\njabber>", -1);
}

void svc_telnet_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    teldata td = (teldata)(ts->data);

    log_debug(ZONE,"svc_telnet_cleanup");
    if(td->s != NULL)
        js_session_end(td->s,"Disconnected");
}

void svc_telnet_read(thread t, void *buffer, int bufsz)
{
    char *eol, *space, *newmsg;
    tstream ts = (tstream)(t->data);
    teldata td = (teldata)(ts->data);

    if(td->initflag)
    {
        pool_cleanup(t->p, svc_telnet_cleanup, (void *)t);
        td->initflag = 0;
    }

    if(bufsz <= 0) return;

    if((eol = strstr(buffer, "\r")) != NULL) *eol = '\0';
    if((eol = strstr(buffer, "\n")) != NULL) *eol = '\0';

    log_debug(ZONE,"phase %d data %s",td->phase, buffer);

    switch(td->phase)
    {
    case PHASE_LOGIN:
        td->login = pstrdup(t->p, buffer);
        tstream_write(t, "password: ", -1);
        td->phase = PHASE_PASS;
        break;
    case PHASE_PASS:
        td->pass = pstrdup(t->p, buffer);
        tstream_write(t, "resource[telnet]: ", -1);
        td->phase = PHASE_RES;
        break;
    case PHASE_RES:
        if(strlen(buffer) > 0)
            td->res = pstrdup(t->p, buffer);
        else
            td->res = pstrdup(t->p, "telnet");

        if(svc_telnet_auth(td))
        {
            tstream_write(t, "Heya bud! :)\n", -1);
        }else if(strcmp(td->res, "register") == 0 && svc_telnet_reg(td) && svc_telnet_auth(td)){
            tstream_write(t, "Account Created\n", -1);
        }else{
            tstream_write(t, "Shea, right!\nlogin: ", -1);
            td->phase = PHASE_LOGIN;
            break;
        }
        /* we're authed! */
        tstream_write(t, "\njabber> ", -1);
        td->phase = PHASE_NORM;
        td->id = jid_new(t->p,xmlnode_get_data(js_config("names/default")));
        jid_set(td->id, td->login, JID_USER);
        jid_set(td->id, td->res, JID_RESOURCE);
        td->s = js_session_new(td->id,svc_telnet_send,(void *)td);
        break;
    case PHASE_NORM:
        space = strstr(buffer," ");
        if(space != NULL)
        {
            *space = '\0';
            ++space;
        }
        if(strcmp(buffer, "quit") == 0)
        {
            tstream_write(t, "Adios Amigo\n\nlogin:", -1);
            js_session_end(td->s,space);
            td->phase = PHASE_LOGIN;
            break;
        }
        if(strcmp(buffer, "message") == 0 && (td->to = jid_new(t->p, space)) != NULL)
        {
            tstream_write(t, "Composing message to ", -1);
            tstream_write(t, space, -1);
            tstream_write(t, ":\n", 2);
            td->phase = PHASE_MSG;
            td->message = malloc(1);
            *(td->message) = '\0';
            break;
        }
        tstream_write(t, "jabber> ", -1);
        break;
    case PHASE_MSG:
        if(strcmp(buffer, ".") == 0)
        {
            svc_telnet_message(td);
            td->phase = PHASE_NORM;
            tstream_write(t, "jabber> ", -1);
            break;
        }
        newmsg = malloc(strlen(td->message) + strlen(buffer) + 2);
        *newmsg = '\0';
        strcat(newmsg,td->message);
        strcat(newmsg," ");
        strcat(newmsg,buffer);
        free(td->message);
        td->message = newmsg;
        break;
    default:
        tstream_write(t, "I suck! I'm dumb!\n", -1);
    }
}

void svc_telnet_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
    teldata newd;
    tstream ts;

    log_debug(ZONE,"svc_telnet connection established (fd: %d, ip: %s, port: %d)\n",
              sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    new = tstream_new(sock, inet_ntoa(sa.sin_addr), svc_telnet_read, NULL, NULL);
    ts = (tstream)(new->data);
    newd = pmalloc(new->p, sizeof(_teldata));
    newd->phase = PHASE_LOGIN;
    newd->initflag = 1;
    newd->t = new;
    newd->s = NULL;
    ts->data = (void *)newd;
    tstream_write(new, "Welcome!\nlogin: ", -1);
}

void svc_telnet()
{
    log_debug(ZONE,"svc_telnet startup");
    js_service_listen("telnet", svc_telnet_connect);
}

/* for fun, 3:15 am Jan 28th 2000:
[jer@atreju jabber-transport]$ telnet localhost 5228
Trying 127.0.0.1...
Connected to localhost.
Escape character is '^]'.
Welcome!
login: jer
password: jabber
resource[telnet]:
Heya bud! :)
jabber> message jer@atreju.jeremie.com/telnet
Composing message to jer@atreju.jeremie.com/telnet:
will?
this be the first 0.9 message? :)
.
jabber>
Incoming Message!
From: jabber:jer@atreju.jeremie.com/telnet
 will? this be the first 0.9 message? :)
jabber>
*/
